/*----------------------------------------------------------------------------

  NLHDR - Non local high dynamic range image generation

  Copyright (c) 2013 cecilia aguerrebere <caguerrebere@gmail.com>
   
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU Affero General Public License for more details.

  This code is partially based on the implementation by A. Buades
  
  A. Buades, B. Coll, and J.-M. Morel. Non-Local Means Denoising. 
  Image Processing On Line, 2011

 ----------------------------------------------------------------------------*/
  

#ifndef _LIBAUXILIAR_H_
#define _LIBAUXILIAR_H_


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>

#define MAX(i,j) ( (i)<(j) ? (j):(i) )
#define MIN(i,j) ( (i)<(j) ? (i):(j) )


#define dTiny 0.00000000000000001f  
#define fTiny 0.00000000001f
#define fLarge 100000000.0f


float fpatch_dist_ch( double * u0, double *u1, double *var0,double *var1, int *satMap0, int *satMap1, int i0, int j0,int i1,int j1,int radius, int width0, int width1, int minPix,float distSatPatchs, int *total_pix_used);
float fpatch_dist(double **u0, double **u1, double **var0, double **var1,int **satMap0,int **satMap1, int i0,int j0,int i1,int j1,int radius,int channels, int width0, int width1, int minPix,float distSatPatchs, int *total_pix_used_out);

void fnormalize_frame_ch( float *u0, double *rad, double *var,double *radDvar, double *invVar, int iWidth, int iHeight, float *prnu, float muR, float sigma2R, float g, float tau );
void fnormalize_frame( float **u0, double **rad, double **var,double **radDvar, double **invVar, int iWidth, int iHeight, float **prnu, float muR, float sigma2R, float *g, float tau, int channels );

void fnormalize_frame_onlyRad_ch( float *u0, double *rad, int iWidth, int iHeight, float *prnu, float muR, float g, float tau );
void fnormalize_frame_onlyRad( float **u0, double **rad, int iWidth, int iHeight, float **prnu, float muR, float *g, float tau, int channels );

void separate_channels( float *u0, float **u1, int ncol, int nrow ); 
void separate_channels_int( float *u0, int **u1, int ncol, int nrow );

void fsaturation_maps_ch( float *u0, int *sat_map, int iWidth, int iHeight, float vsat );
void fsaturation_maps( float **u0, int **sat_map, int iWidth, int iHeight, int channels, float vsat );

void remake_bayer( float **u1, float *u0, int ncol_ch, int nrow_ch );

void estimate_translation( double *img_ref, double *img, int *satMap_ref, int *satMap, int iWidth, int iHeight,int y_min, int y_max,int x_min, int x_max, int* trans_y_out, int* trans_x_out );
void global_transl_estimations( float ***fpI, int iFrames, int iChannels, int ref_t, int iwxh, int iWidth, int iHeight, float ** prnu, float muR, float* g, float iVsat, float* tau, int y_min, int y_max, int x_min, int x_max, int *trans_y_out, int* trans_x_out);

int total_valid_pixels( int *satMap_common, int yt, int xt, int iHeight_t, int iWidth_t, int iWidth);

void error(const char * msg);
void print_info ( int win, int bloc, float ih, int ref_t, int iFrames, float* tau, int d_c,	float* g, float muR, float sigma2R, float iVsat, char** input, int* trans_y, int* trans_x, const char* prnu );

#endif
